@@ -72,6 +72,7 @@ struct fio_file {
 	 */
 	char *file_name;
 	unsigned int major, minor;
+	int fileno;
 
 	void *mmap_ptr;
 	size_t mmap_sz;
@@ -146,6 +147,7 @@ FILE_FLAG_FNS(partial_mmap);
 struct thread_data;
 extern void close_files(struct thread_data *);
 extern void close_and_free_files(struct thread_data *);
+extern unsigned long long get_start_offset(struct thread_data *);
 extern int __must_check setup_files(struct thread_data *);
 extern int __must_check file_invalidate_cache(struct thread_data *, struct fio_file *);
 extern int __must_check generic_open_file(struct thread_data *, struct fio_file *);
