@@ -23,16 +23,7 @@
 
 #include "lib/getopt.h"
 
-#include "fio_version.h"
-
-#if FIO_PATCH > 0
-const char fio_version_string[] =	__fio_stringify(FIO_MAJOR) "."	\
-					__fio_stringify(FIO_MINOR) "."	\
-					__fio_stringify(FIO_PATCH);
-#else
-const char fio_version_string[] =	__fio_stringify(FIO_MAJOR) "."	\
-					__fio_stringify(FIO_MINOR);
-#endif
+const char fio_version_string[] = FIO_VERSION;
 
 #define FIO_RANDSEED		(0xb1899bedUL)
 
@@ -816,7 +807,7 @@ static int add_job(struct thread_data *td, const char *jobname, int job_add_num)
 			f->real_file_size = -1ULL;
 	}
 
-	td->mutex = fio_mutex_init(0);
+	td->mutex = fio_mutex_init(FIO_MUTEX_LOCKED);
 
 	td->ts.clat_percentiles = td->o.clat_percentiles;
 	if (td->o.overwrite_plist)
@@ -1188,7 +1179,7 @@ static int fill_def_thread(void)
 
 static void usage(const char *name)
 {
-	printf("fio %s\n", fio_version_string);
+	printf("%s\n", fio_version_string);
 	printf("%s [options] [job options] <job file(s)>\n", name);
 	printf("  --debug=options\tEnable debug logging. May be one/more of:\n"
 		"\t\t\tprocess,file,io,mem,blktrace,verify,random,parse,\n"
@@ -1419,7 +1410,7 @@ int parse_cmd_line(int argc, char *argv[])
 			break;
 		case 'v':
 			if (!cur_client) {
-				log_info("fio %s\n", fio_version_string);
+				log_info("%s\n", fio_version_string);
 				do_exit++;
 			}
 			break;
@@ -1645,7 +1636,7 @@ int parse_options(int argc, char *argv[])
 	}
 
 	if (!terse_output)
-		log_info("fio %s\n", fio_version_string);
+		log_info("%s\n", fio_version_string);
 
 	return 0;
 }
